/*
 * JBoss, Home of Professional Open Source
 * Copyright (c) 2010, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.ejb3.interceptors.test.ejbthree2080;

import javax.annotation.PostConstruct;
import javax.ejb.Stateful;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import java.util.concurrent.atomic.AtomicInteger;

import static org.jboss.ejb3.interceptors.test.ejbthree2080.StateMachineLocal.State.*;

/**
 * @author <a href="cdewolf@redhat.com">Carlo de Wolf</a>
 */
@Stateful
public class MyStateMachine implements StateMachineLocal
{
   private static AtomicInteger invocations = new AtomicInteger(0);
   private State state;

   /**
    * To make sure InjectInterceptorsFactory is in there.
    */
   @AroundInvoke
   public Object aroundInvoke(InvocationContext ctx) throws Exception
   {
      invocations.incrementAndGet();
      return ctx.proceed();
   }

   @PostConstruct
   protected void initialize()
   {
      this.state = INITIALIZED;
   }

   public static int invocations()
   {
      return invocations.getAndSet(0);
   }

   public static String mapState(State state)
   {
      return state.name();
   }

   public void start()
   {
      this.state = STARTED;
   }

   public void stop()
   {
      this.state = STOPPED;
   }
}
