/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2007, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
  *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.ejb3.test.interceptors.common.aop;

import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.logging.Logger;

/**
 * Log as much as we can about the invocation, then proceed.
 *
 * @author <a href="mailto:carlo.dewolf@jboss.com">Carlo de Wolf</a>
 * @version $Revision: 72280 $
 */
public class LoggingInterceptor implements Interceptor
{
   private static final Logger log = Logger.getLogger(LoggingInterceptor.class);
   
   public String getName()
   {
      return "LoggingInterceptor";
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      log.info("invoke " + invocation);
      log.info("  advisor " + invocation.getAdvisor().getName());
      log.info("  target object " + invocation.getTargetObject());
      //log.info("  " + invocation.getResponseContextInfo());
      return invocation.invokeNext();
   }

}
